/******************************************************************************
* OpenLP - Open Source Lyrics Projection                                      *
* --------------------------------------------------------------------------- *
* Copyright (c) 2008-2016 OpenLP Developers                                   *
* --------------------------------------------------------------------------- *
* This program is free software; you can redistribute it and/or modify it     *
* under the terms of the GNU General Public License as published by the Free  *
* Software Foundation; version 2 of the License.                              *
*                                                                             *
* This program is distributed in the hope that it will be useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       *
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    *
* more details.                                                               *
*                                                                             *
* You should have received a copy of the GNU General Public License along     *
* with this program; if not, write to the Free Software Foundation, Inc., 59  *
* Temple Place, Suite 330, Boston, MA 02111-1307 USA                          *
******************************************************************************/

body {
  background-color: black;
  font-family: sans-serif;
  overflow: hidden;
}

#currentslide {
  font-size: 40pt;
  color: white;
  padding-bottom: 0px;
}

#nextslide {
  font-size: 40pt;
  color: grey;
  padding-top: 0px;
  padding-bottom: 0px;
}

#right {
  float: right;
}

#clock {
  font-size: 30pt;
  color: yellow;
  text-align: right;
}

#notes {
  font-size: 36pt;
  color: salmon;
  text-align: right;
}

#verseorder {
  font-size: 30pt;
  color: green;
  text-align: left;
}

.currenttag {
   color: lightgreen;
   font-weight: bold;
}
